#include <windows.h>
#include <mmsystem.h>
#include <stdio.h>

#pragma comment(lib, "winmm.lib")

DWORD CPUIDMax    = 0;
DWORD CPUIDExtMax = 0;

char szVenderID[13] = {0};
char szCpuName[49] = {0};

DWORD TypeID     = 0;
DWORD FamilyID   = 0;
DWORD ModelID    = 0;
DWORD SteppingID = 0;

DWORD hasCPUID   = 0;
DWORD hasFPU     = 0;
DWORD hasTSC     = 0;
DWORD hasMSR     = 0;
DWORD hasCMOV    = 0;
DWORD hasMMX     = 0;
DWORD has3DNow   = 0;

DWORD hasSSE_INT = 0;
DWORD hasSSE_FP  = 0;
DWORD has3DNowExt= 0;

float fCpuClock  = 0;


void GetCpuID(){
	__asm{
		//Check CPUID
		pushfd
		pop eax
		mov ebx, eax
		xor eax, 1<<21
		push eax
		popfd
		pushfd
		pop eax
		cmp eax, ebx
		je exitasm
		mov [hasCPUID], 1

		//CPUID 0
		xor eax, eax
		cpuid
		mov [CPUIDMax], eax
		mov DWORD PTR [szVenderID+0], ebx
		mov DWORD PTR [szVenderID+4], edx
		mov DWORD PTR [szVenderID+8], ecx
		mov BYTE PTR [szVenderID+12], 0

		//CPUID 1
		cmp eax, 1
		jb exitasm
		mov eax, 1
		cpuid

		mov esi, eax
		and esi, 0xf
		mov [SteppingID], esi

		shr eax, 4
		mov esi, eax
		and esi, 0xf
		mov [ModelID], esi

		shr eax, 4
		mov esi, eax
		and esi, 0xf
		mov [FamilyID], esi

		shr eax, 4
		mov esi, eax
		and esi, 0x3
		mov [TypeID], esi


		xor eax, eax
		test edx, 1<<0
		setnz al
		mov [hasFPU], eax

		test edx, 1<<4
		setnz al
		mov [hasTSC], eax

		test edx, 1<<5
		setnz al
		mov [hasMSR], eax

		test edx, 1<<15
		setnz al
		mov [hasCMOV], eax

		test edx, 1<<23
		setnz al
		mov [hasMMX], eax

		test edx, 1<<25
		setnz al
		mov [hasSSE_INT], eax
		mov [hasSSE_FP], eax

		//CPUID 0x80000000
		mov eax, 0x80000000
		cpuid
		mov [CPUIDExtMax], eax

		//CPUID 0x80000001
		cmp eax, 0x80000001
		jb exitasm
		mov eax, 0x80000001
		cpuid

		xor eax, eax
		test edx, 1<<22
		setnz al
		mov [hasSSE_INT], eax

		test edx, 1<<30
		setnz al
		mov [has3DNowExt], eax

		test edx, 1<<31
		setnz al
		mov [has3DNow], eax

		//CPUID 0x80000002-4
		cmp [CPUIDExtMax], 0x80000004
		jb exitasm
		mov eax, 0x80000002
		cpuid
		mov DWORD PTR [szCpuName+ 0], eax
		mov DWORD PTR [szCpuName+ 4], ebx
		mov DWORD PTR [szCpuName+ 8], ecx
		mov DWORD PTR [szCpuName+12], edx
		mov eax, 0x80000003
		cpuid
		mov DWORD PTR [szCpuName+16], eax
		mov DWORD PTR [szCpuName+20], ebx
		mov DWORD PTR [szCpuName+24], ecx
		mov DWORD PTR [szCpuName+28], edx
		mov eax, 0x80000004
		cpuid
		mov DWORD PTR [szCpuName+32], eax
		mov DWORD PTR [szCpuName+36], ebx
		mov DWORD PTR [szCpuName+40], ecx
		mov DWORD PTR [szCpuName+44], edx
		

	exitasm:
	}
}

void GetCpuClock(){
	DWORD dwCpuClock;
	DWORD dwMsCount;

	if (hasTSC == 0)
		return;

	__asm{
		rdtsc
		mov [dwCpuClock], eax
	}

	dwMsCount = timeGetTime();
	while(timeGetTime() - dwMsCount < 500);

	__asm{
		rdtsc
		sub eax, [dwCpuClock]
		mov [dwCpuClock], eax
	}

	fCpuClock = (float)(dwCpuClock/500000.0);
}

void main(){
	GetCpuID();
	GetCpuClock();

	printf("VenderID  %s\n", szVenderID);
	printf("CpuName   %s\n", szCpuName);
	printf("TypeID %d, FamilyID %d, ModelID %d, SteppingID %d\n", TypeID, FamilyID, ModelID, SteppingID);
	printf("CpuClock %.1fMHz\n", fCpuClock);

	printf("\n");
	printf("CPUID       %d\n", hasCPUID);		//CPUIDT|[g̗L
	printf("CPUIDMax    %x\n", CPUIDMax);		//CPUIDW@\̃T|[gől
	printf("CPUIDExtMax %x\n", CPUIDExtMax);	//CPUIDg@\̃T|[gől

	printf("FPU    %d\n", hasFPU);				//_Z̗L
	printf("TSC    %d\n", hasTSC);				//RDTSC߂̗L
	printf("MSR    %d\n", hasMSR);				//RDMSR,WRMSR߂̗L
	printf("CMOV   %d\n", hasCMOV);				//CMOV߂̗L
	printf("MMX    %d\n", hasMMX);				//MMX߂̗L
	printf("3DNow! %d\n", has3DNow);			//3DNow!߂̗L

	printf("SSE_INT    %d\n", hasSSE_INT);		//SSE integer(=MMXg)߂̗L
	printf("SSE_FP     %d\n", hasSSE_FP);		//SSE floating-point߂̗L
	printf("3DNow! Ext %d\n", has3DNowExt);		//3DNow!g߂̗L
	printf("FCMOV      %d\n", hasFPU & hasCMOV);//FCMOV߂̗L

	getchar();
}